package jp.co.intra_mart.cookbook_114393;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import jp.co.intra_mart.foundation.viewcreator.DataViewManager;
import jp.co.intra_mart.foundation.viewcreator.exception.FunctionExecuteException;
import jp.co.intra_mart.foundation.viewcreator.exception.QueryException;
import jp.co.intra_mart.foundation.viewcreator.formula.FunctionImplementation;
import jp.co.intra_mart.foundation.viewcreator.formula.ViewCreatorData;
import jp.co.intra_mart.foundation.viewcreator.model.DataView;
import jp.co.intra_mart.foundation.viewcreator.model.ListColumn;
import jp.co.intra_mart.foundation.viewcreator.model.ListDataView;
import jp.co.intra_mart.foundation.viewcreator.model.VCQuery;
import jp.co.intra_mart.foundation.viewcreator.model.formula.Arguments;
import jp.co.intra_mart.foundation.viewcreator.model.formula.FixedNumber;
import jp.co.intra_mart.foundation.viewcreator.model.formula.FixedValue;
import jp.co.intra_mart.foundation.viewcreator.model.formula.funcimpl.Only4RDBFunction;
import jp.co.intra_mart.foundation.viewcreator.user.ListDataViewParameters;
import jp.co.intra_mart.vc_tm.common.VCTMConnectionManager;

public class FunctionCurrentCount implements FunctionImplementation, Only4RDBFunction {

    private ViewCreatorData data;

    @Override
    public void init(final ViewCreatorData data) {
        this.data = data;

        // 全件取得するために、リスト集計でグループ化がセットされている場合は解除しておく
        final ListDataView view = data.getView();

        if (view.getPattern() == DataView.PATTERN_LIST) {
            for (final ListColumn listColumn : view.getColumns()) {
                if (listColumn.getType() == ListColumn.TYPE_GROUP) {
                    // タイプの設定を無しにする
                    listColumn.setType(ListColumn.TYPE_NONE);
                }
            }
        } else {
            // サマリ集計の場合は、グループ化の設定を解かないように上記処理をスキップします
        }
    }

    @Override
    public String getName() {
        // 関数名
        return "CURRENTCOUNT";
    }

    @Override
    public FixedValue execute(final Arguments args) throws FunctionExecuteException {

        // 画面から入力された検索条件やソートの状態を取得
        final ListDataViewParameters params = this.data.getParams();
        // データ参照の定義情報を取得
        final ListDataView view = this.data.getView();
        // クエリの定義情報を取得
        final VCQuery query = this.data.getQuery();

        // 画面からの入力値や定義情報を元に全件取得するSQLを生成
        final String selectSql;
        try {
            selectSql = DataViewManager.generateSql4List(params, view, query, false);
        } catch (final QueryException | SQLException e) {
            throw new FunctionExecuteException(e);
        }

        // 生成されたSQLをCOUNT関数のSELECTで包む
        final StringBuilder countSql = new StringBuilder("");
        countSql.append("SELECT COUNT(*) AS CNT FROM (");
        countSql.append(selectSql);
        countSql.append(") _IM_VC_COOKBOOK_114393");

        // SQLを実行しCOUNT関数の結果を取得する
        final int currentCount;
        try (final Connection con = VCTMConnectionManager.getConnection(query.getDataSource());
                final PreparedStatement stmt = con.prepareStatement(countSql.toString())) {
            final ResultSet rs = stmt.executeQuery();
            if (rs.next()) {
                currentCount = rs.getInt("CNT");
            } else {
                throw new FunctionExecuteException("fail");
            }
        } catch (final SQLException e) {
            throw new FunctionExecuteException(e);
        }

        // COUNT関数の結果を数値型で返却
        return new FixedNumber(currentCount);
    }

}
